import workflow
import os

ICON_BASE_DIR = os.path.abspath(os.path.dirname(__file__)) + '/static'

ICON_DEFAULT = workflow.ICON_INFO
ICON_TIPS = os.path.join(ICON_BASE_DIR, 'tips.png')
ICON_WARNING = os.path.join(ICON_BASE_DIR, 'warning.png')
ICON_ERROR = workflow.ICON_ERROR


def show_tips(wf, msg):
    wf.add_item(msg, '', arg='', valid=True, icon=ICON_TIPS)


def show_warning(wf, msg):
    wf.add_item(msg, '', arg='', valid=True, icon=ICON_WARNING)


def show_error(wf, msg, content=''):
    wf.add_item(msg, content, arg='', valid=True, icon=ICON_ERROR)



from sample import SampleWorkflow

conf = {
    'sample': SampleWorkflow
}


def register_config(new_conf):
    global conf
    conf = new_conf


def search_handler(cmd):
    commands = []
    if cmd == 'all' or cmd == 'ALL':
        return conf.keys(), True

    if cmd in conf:
        commands.append(cmd)
        return commands, False

    for each in conf.keys():
        if each.startswith(cmd):
            commands.append(each)
    return commands, True


def get_handler(name):
    if name not in conf:
        raise Exception('cannot found handler: {}'.format(name))
    return conf[name]


def get_handle_size():
    return len(conf)



import common
import sys
import registry

reload(sys)
sys.setdefaultencoding('utf8')


def __execute(wf, cmd, params):
    logger = wf.logger

    commands, fuzzy = registry.search_handler(cmd)
    logger.info('commands: {}'.format(commands))
    if len(commands) == 0:
        return common.show_warning(wf, 'Cannot found command: {}'.format(cmd))

    if fuzzy and registry.get_handle_size() > 1:
        for c in commands:
            handle = registry.get_handler(c)
            instance = handle(wf, params)
            wf.add_item(c, instance.get_desc(), autocomplete=c, icon=instance.get_icon_path())
        return

    handler = registry.get_handler(commands[0])
    instance = handler(wf, params)
    instance.execute()


def main(wf):
    logger = wf.logger
    args = sys.argv
    logger.info('input args: {}'.format(sys.argv))

    if len(args) < 2:
        common.show_tips(wf, 'The command is required')
        wf.send_feedback()
        return

    cmd = args[1]
    params = []
    if len(args) >= 2:
        params = args[2:]

    logger.info('cmd: {}, params: {}'.format(cmd, params))
    try:
        __execute(wf, cmd, params)
    except Exception as e:
        logger.exception(e)
        common.show_error(wf, 'Fail to execute handler', str(e))
    finally:
        wf.send_feedback()



from base import TwoParamWorkflow
import common


class SampleWorkflow(TwoParamWorkflow):

    def __init__(self, wf, params):
        super(SampleWorkflow, self).__init__(wf, params)

    def execute_with_two_param(self, v1, v2):
        self.add_item('{} {}'.format(v1, v2))

    def get_icon_path(self):
        return common.ICON_DEFAULT



# encoding:utf-8
from setuptools import setup

setup(
    name='alfred-workflow-flyer',
    version='0.0.1',
    author='Pushy',
    author_email='1437876073@qq.com',
    url='',
    description=u'A simple scaffold of alfred workflow',
    packages=['workflow_flyer'],
    install_requires=['Alfred-Workflow']
)



import common
from workflow.notify import notify as wf_notify


class BaseWorkflow(object):

    def __init__(self, wf, params):
        self.wf = wf
        self.params = params

    def execute(self):
        self.show_tips('Hello World')

    def add_item(self, title, sub_title=None, copy_text=None, autocomplete=None):
        self.wf.add_item(
            title=title,
            subtitle=sub_title or '',
            arg=copy_text or '',
            autocomplete=autocomplete,
            valid=True,
            icon=self.get_icon_path()
        )

    def show_tips(self, msg):
        common.show_tips(self.wf, msg)

    def show_warning(self, msg):
        common.show_warning(self.wf, msg)

    def show_error(self, msg, content=''):
        common.show_error(self.wf, msg, content)

    @staticmethod
    def notify(title, content, sound=None):
        wf_notify(title, content, sound=sound)

    def logger(self):
        return self.wf.logger

    def get_icon_path(self):
        return common.ICON_DEFAULT

    def get_desc(self):
        return 'No any description'


class SingleParamWorkflow(BaseWorkflow):

    def __init__(self, wf, params):
        super(SingleParamWorkflow, self).__init__(wf, params)

    def execute(self):
        if len(self.params) < 1:
            self.show_tips('The value is required')
            return

        first_param = self.params[0]
        if not first_param:
            self.show_tips('The first param is empty')
            return
        self.execute_with_single_param(first_param)

    def execute_with_single_param(self, value):
        raise Exception('unsupported method')


class TwoParamWorkflow(BaseWorkflow):

    def __init__(self, wf, params):
        super(TwoParamWorkflow, self).__init__(wf, params)

    def execute(self):
        if len(self.params) < 1:
            self.show_tips('The value is required')
            return

        if len(self.params) < 2:
            self.show_tips('The second param is required')
            return

        first_param = self.params[0]
        second_param = self.params[1]
        if not first_param or not second_param:
            self.show_tips('The param is empty')
            return
        self.execute_with_two_param(first_param, second_param)

    def execute_with_two_param(self, v1, v2):
        raise Exception('unsupported method')


class ThirdParamWorkflow(BaseWorkflow):
    def __init__(self, wf, params):
        super(ThirdParamWorkflow, self).__init__(wf, params)

    def execute(self):
        if len(self.params) < 1:
            self.show_tips('The value is required')
            return

        if len(self.params) < 2:
            self.show_tips('The second param is required')
            return

        if len(self.params) < 3:
            self.show_tips('The third param is required')
            return

        first_param = self.params[0]
        second_param = self.params[1]
        third_param = self.params[2]
        if not first_param or not second_param or not third_param:
            self.show_tips('The param is empty')
            return
        self.execute_with_third_param(first_param, second_param, third_param)

    def execute_with_third_param(self, v1, v2, v3):
        raise Exception('unsupported method')
